package org.apache.bcel.util;

import java.io.InputStream;
import java.io.IOException;
import org.apache.bcel.classfile.ClassParser;
import java.util.HashMap;
import org.apache.bcel.classfile.JavaClass;
import java.util.Map;

public class ClassLoaderRepository implements Repository
{
    private final ClassLoader loader;
    private final Map<String, JavaClass> loadedClasses;
    
    public ClassLoaderRepository(final ClassLoader loader) {
        this.loadedClasses = new HashMap<String, JavaClass>();
        this.loader = loader;
    }
    
    @Override
    public void clear() {
        this.loadedClasses.clear();
    }
    
    @Override
    public JavaClass findClass(final String className) {
        return this.loadedClasses.get(className);
    }
    
    @Override
    public ClassPath getClassPath() {
        return null;
    }
    
    @Override
    public JavaClass loadClass(final Class<?> clazz) throws ClassNotFoundException {
        return this.loadClass(clazz.getName());
    }
    
    @Override
    public JavaClass loadClass(final String className) throws ClassNotFoundException {
        final String classFile = className.replace('.', '/');
        JavaClass RC = this.findClass(className);
        if (RC != null) {
            return RC;
        }
        try (InputStream is = this.loader.getResourceAsStream(classFile + ".class")) {
            if (is == null) {
                throw new ClassNotFoundException(className + " not found.");
            }
            final ClassParser parser = new ClassParser(is, className);
            RC = parser.parse();
            this.storeClass(RC);
            return RC;
        }
        catch (final IOException e) {
            throw new ClassNotFoundException(className + " not found: " + e, e);
        }
    }
    
    @Override
    public void removeClass(final JavaClass clazz) {
        this.loadedClasses.remove(clazz.getClassName());
    }
    
    @Override
    public void storeClass(final JavaClass clazz) {
        this.loadedClasses.put(clazz.getClassName(), clazz);
        clazz.setRepository(this);
    }
}
